@@ -90,11 +90,15 @@
 
 #include <glibmm/arrayhandle.h>
 #include <glibmm/balancedtree.h>
+#include <glibmm/base64.h>
+#include <glibmm/bytearray.h>
+#include <glibmm/bytes.h>
 #include <glibmm/checksum.h>
 #include <glibmm/class.h>
 #include <glibmm/containerhandle_shared.h>
 #include <glibmm/convert.h>
 #include <glibmm/date.h>
+#include <glibmm/datetime.h>
 #include <glibmm/dispatcher.h>
 #include <glibmm/error.h>
 #include <glibmm/exception.h>
@@ -132,11 +136,13 @@
 #include <glibmm/threadpool.h>
 #include <glibmm/timer.h>
 #include <glibmm/timeval.h>
+#include <glibmm/timezone.h>
 #include <glibmm/uriutils.h>
 #include <glibmm/ustring.h>
 #include <glibmm/value.h>
 #include <glibmm/valuearray.h>
 #include <glibmm/variant.h>
+#include <glibmm/variantdict.h>
 #include <glibmm/variantiter.h>
 #include <glibmm/varianttype.h>
 #include <glibmm/vectorutils.h>
