@@ -19,15 +19,54 @@
  */
 
 #undef G_DISABLE_DEPRECATED //So we can use newly-deprecated API, to preserve our API.
+#define GLIB_DISABLE_DEPRECATION_WARNINGS 1
+
+#include <glibmm/threads.h>
+
+#ifndef GLIBMM_DISABLE_DEPRECATED
+#include <glibmm/thread.h>
+#endif //GLIBMM_DISABLE_DEPRECATED
+
 #include <glibmm/main.h>
 #include <glibmm/exceptionhandler.h>
-#include <glibmm/thread.h>
 #include <glibmm/wrap.h>
 #include <glibmm/iochannel.h>
 #include <algorithm>
+#include <map> // Needed until the next ABI break.
 
 namespace
 {
+#ifdef GLIBMM_DISABLE_DEPRECATED
+void time64_to_time_val(gint64 time64, Glib::TimeVal& time_val)
+{
+  // This function is not guaranteed to convert correctly if time64 is negative.
+  const long seconds = static_cast<long>(time64 / G_GINT64_CONSTANT(1000000));
+  const long microseconds = static_cast<long>(time64 -
+                            static_cast<gint64>(seconds) * G_GINT64_CONSTANT(1000000));
+  time_val = Glib::TimeVal(seconds, microseconds);
+}
+#endif //GLIBMM_DISABLE_DEPRECATED
+
+//TODO: At the next ABI break, replace ExtraSourceData by new data members in Source.
+// Then the mutex is not necessary, but to keep the code thread-safe, use the
+// g_atomic_*() functions on these data elements.
+// These are new data members that can't be added to Glib::Source now,
+// because it would break ABI.
+struct ExtraSourceData
+{
+  ExtraSourceData()
+  : ref_count(1), keep_wrapper(2)
+  {}
+  int ref_count;
+  // When both Source::unreference() and SourceCallbackData::destroy_notify_callback()
+  // have decreased keep_wrapper, it's time to delete the C++ wrapper.
+  int keep_wrapper;
+};
+
+std::map<const Glib::Source*, ExtraSourceData> extra_source_data;
+// Source instances may be used in different threads.
+// Accesses to extra_source_data must be thread-safe.
+Glib::Threads::Mutex extra_source_data_mutex;
 
 class SourceConnectionNode
 {
@@ -67,7 +106,7 @@ void* SourceConnectionNode::notify(void* data)
     g_source_destroy(s);
 
     // Destroying the object triggers execution of destroy_notify_handler(),
-    // eiter immediately or later, so we leave that to do the deletion.
+    // either immediately or later, so we leave that to do the deletion.
   }
 
   return 0;
@@ -103,8 +142,8 @@ sigc::slot_base* SourceConnectionNode::get_slot()
 /* We use the callback data member of GSource to store both a pointer to our
  * wrapper and a pointer to the connection node that is currently being used.
  * The one and only SourceCallbackData object of a Glib::Source is constructed
- * in the ctor of Glib::Source and destroyed after the GSource object when the
- * reference counter of the GSource object reaches zero!
+ * in the ctor of Glib::Source and destroyed when the GSource object is destroyed,
+ * which may occur before the reference counter of the GSource object reaches zero!
  */
 struct SourceCallbackData
 {
@@ -142,7 +181,16 @@ void SourceCallbackData::destroy_notify_callback(void* data)
     SourceConnectionNode::destroy_notify_callback(self->node);
 
   if(self->wrapper)
-    Glib::Source::destroy_notify_callback(self->wrapper);
+  {
+    Glib::Threads::Mutex::Lock lock(extra_source_data_mutex);
+    if (--extra_source_data[self->wrapper].keep_wrapper == 0)
+    {
+      // No other reference exists to the wrapper. Delete it!
+      extra_source_data.erase(self->wrapper);
+      lock.release();
+      Glib::Source::destroy_notify_callback(self->wrapper);
+    }
+  }
 
   delete self;
 }
@@ -152,7 +200,7 @@ void SourceCallbackData::destroy_notify_callback(void* data)
  */
 static SourceCallbackData* glibmm_source_get_callback_data(GSource* source)
 {
-  g_return_val_if_fail(source->callback_funcs->get != 0, 0);
+  g_return_val_if_fail(source->callback_funcs != 0, 0);
 
   GSourceFunc func;
   void* user_data = 0;
@@ -177,9 +225,9 @@ static gboolean glibmm_dummy_source_callback(void*)
   return 0;
 }
 
-/* Only used by SignalTimeout::connect() and SignalIdle::connect().
- * These don't use Glib::Source, to avoid the unnecessary overhead
- * of a completely unused wrapper object.
+/* Only used by SignalTimeout::connect(), SignalTimeout::connect_seconds()
+ * and SignalIdle::connect(). These don't use Glib::Source, to avoid the
+ * unnecessary overhead of a completely unused wrapper object.
  */
 static gboolean glibmm_source_callback(void* data)
 {
@@ -197,6 +245,26 @@ static gboolean glibmm_source_callback(void* data)
   return 0;
 }
 
+/* Only used by SignalTimeout::connect_once(), SignalTimeout::connect_seconds_once()
+ * and SignalIdle::connect_once(). These don't use Glib::Source, to avoid the
+ * unnecessary overhead of a completely unused wrapper object.
+ */
+static gboolean glibmm_source_callback_once(void* data)
+{
+  SourceConnectionNode* const conn_data = static_cast<SourceConnectionNode*>(data);
+
+  try
+  {
+    // Recreate the specific slot from the generic slot node.
+    (*static_cast<sigc::slot<void>*>(conn_data->get_slot()))();
+  }
+  catch(...)
+  {
+    Glib::exception_handlers_invoke();
+  }
+  return 0; // Destroy the event source after one call
+}
+
 static gboolean glibmm_iosource_callback(GIOChannel*, GIOCondition condition, void* data)
 {
   SourceCallbackData *const callback_data = static_cast<SourceCallbackData*>(data);
@@ -234,6 +302,45 @@ static gboolean glibmm_child_watch_callback(GPid pid, gint child_status, void* d
   return 0;
 }
 
+static void glibmm_signal_connect_once(const sigc::slot<void>& slot, int priority,
+                                       GSource* source, GMainContext* context)
+{
+  SourceConnectionNode* const conn_node = new SourceConnectionNode(slot);
+
+  if (priority != G_PRIORITY_DEFAULT)
+    g_source_set_priority(source, priority);
+
+  g_source_set_callback(
+    source, &glibmm_source_callback_once, conn_node,
+    &SourceConnectionNode::destroy_notify_callback);
+
+  conn_node->install(source);
+  g_source_attach(source, context);
+  g_source_unref(source); // GMainContext holds a reference
+}
+
+gboolean glibmm_main_context_invoke_callback(void* data)
+{
+  sigc::slot_base *const slot = reinterpret_cast<sigc::slot_base*>(data);
+
+  try
+  {
+    // Recreate the specific slot from the generic slot node.
+    return (*static_cast<sigc::slot<bool>*>(slot))();
+  }
+  catch(...)
+  {
+    Glib::exception_handlers_invoke();
+  }
+  return 0;
+}
+
+void glibmm_main_context_invoke_destroy_notify_callback(void* data)
+{
+  sigc::slot_base *const slot = reinterpret_cast<sigc::slot_base*>(data);
+  delete slot;
+}
+
 } // anonymous namespace
 
 
@@ -288,17 +395,18 @@ sigc::connection SignalTimeout::connect(const sigc::slot<bool>& slot,
       source, &glibmm_source_callback, conn_node,
       &SourceConnectionNode::destroy_notify_callback);
 
+  conn_node->install(source);
   g_source_attach(source, context_);
   g_source_unref(source); // GMainContext holds a reference
 
-  conn_node->install(source);
   return connection;
 }
 
 void SignalTimeout::connect_once(const sigc::slot<void>& slot,
                                  unsigned int interval, int priority)
 {
-    connect(sigc::bind_return(slot, false), interval, priority);
+  GSource* const source = g_timeout_source_new(interval);
+  glibmm_signal_connect_once(slot, priority, source, context_);
 }
 
 /* Note that this is our equivalent of g_timeout_add_seconds(). */
@@ -317,17 +425,18 @@ sigc::connection SignalTimeout::connect_seconds(const sigc::slot<bool>& slot,
       source, &glibmm_source_callback, conn_node,
       &SourceConnectionNode::destroy_notify_callback);
 
+  conn_node->install(source);
   g_source_attach(source, context_);
   g_source_unref(source); // GMainContext holds a reference
 
-  conn_node->install(source);
   return connection;
 }
 
 void SignalTimeout::connect_seconds_once(const sigc::slot<void>& slot,
                                          unsigned int interval, int priority)
 {
-    connect_seconds(sigc::bind_return(slot, false), interval, priority);
+  GSource* const source = g_timeout_source_new_seconds(interval);
+  glibmm_signal_connect_once(slot, priority, source, context_);
 }
 
 SignalTimeout signal_timeout()
@@ -358,16 +467,17 @@ sigc::connection SignalIdle::connect(const sigc::slot<bool>& slot, int priority)
       source, &glibmm_source_callback, conn_node,
       &SourceConnectionNode::destroy_notify_callback);
 
+  conn_node->install(source);
   g_source_attach(source, context_);
   g_source_unref(source); // GMainContext holds a reference
 
-  conn_node->install(source);
   return connection;
 }
 
 void SignalIdle::connect_once(const sigc::slot<void>& slot, int priority)
 {
-    connect(sigc::bind_return(slot, false), priority);
+  GSource* const source = g_idle_source_new();
+  glibmm_signal_connect_once(slot, priority, source, context_);
 }
 
 SignalIdle signal_idle()
@@ -443,10 +553,10 @@ sigc::connection SignalChildWatch::connect(const sigc::slot<void, GPid, int>& sl
       source, (GSourceFunc)&glibmm_child_watch_callback, conn_node,
       &SourceConnectionNode::destroy_notify_callback);
 
+  conn_node->install(source);
   g_source_attach(source, context_);
   g_source_unref(source); // GMainContext holds a reference
 
-  conn_node->install(source);
   return connection;
 }
 
@@ -489,10 +599,17 @@ bool MainContext::acquire()
   return g_main_context_acquire(gobj());
 }
 
+#ifndef GLIBMM_DISABLE_DEPRECATED
 bool MainContext::wait(Glib::Cond& cond, Glib::Mutex& mutex)
 {
   return g_main_context_wait(gobj(), cond.gobj(), mutex.gobj());
 }
+#endif //GLIBMM_DISABLE_DEPRECATED
+
+bool MainContext::wait(Glib::Threads::Cond& cond, Glib::Threads::Mutex& mutex)
+{
+  return g_main_context_wait(gobj(), cond.gobj(), mutex.gobj());
+}
 
 void MainContext::release()
 {
@@ -560,6 +677,15 @@ void MainContext::remove_poll(PollFD& fd)
   g_main_context_remove_poll(gobj(), fd.gobj());
 }
 
+void MainContext::invoke(const sigc::slot<bool>& slot, int priority)
+{
+  // Make a copy of slot on the heap.
+  sigc::slot_base* const slot_copy = new sigc::slot<bool>(slot);
+
+  g_main_context_invoke_full(gobj(), priority, glibmm_main_context_invoke_callback,
+    slot_copy, glibmm_main_context_invoke_destroy_notify_callback);
+}
+
 SignalTimeout MainContext::signal_timeout()
 {
   return SignalTimeout(gobj());
@@ -757,12 +883,35 @@ GSource* Source::gobj_copy() const
 
 void Source::reference() const
 {
-  g_source_ref(gobject_);
+  Glib::Threads::Mutex::Lock lock(extra_source_data_mutex);
+  ++extra_source_data[this].ref_count;
 }
 
 void Source::unreference() const
 {
-  g_source_unref(gobject_);
+  Glib::Threads::Mutex::Lock lock(extra_source_data_mutex);
+  if (--extra_source_data[this].ref_count == 0)
+  {
+    GSource* const tmp_gobject = gobject_;
+
+    if (--extra_source_data[this].keep_wrapper == 0)
+    {
+      // The last reference from a RefPtr<Source> has been deleted, and
+      // SourceCallbackData::destroy_notify_callback() has been called while
+      // extra_source_data[this].keep_wrapper was > 1.
+      // Delete the wrapper!
+      extra_source_data.erase(this);
+      lock.release();
+      destroy_notify_callback(const_cast<Source*>(this));
+    }
+    else
+      lock.release();
+
+    // Drop the one and only GSource reference held by the C++ wrapper.
+    // If the GSource instance is attached to a main context, the GMainContext
+    // holds a reference until the source is detached (destroyed).
+    g_source_unref(tmp_gobject);
+  }
 }
 
 Source::Source()
@@ -771,7 +920,7 @@ Source::Source()
 {
   g_source_set_callback(
       gobject_, &glibmm_dummy_source_callback,
-      new SourceCallbackData(this), // our persistant callback data object
+      new SourceCallbackData(this), // our persistent callback data object
       &SourceCallbackData::destroy_notify_callback);
 }
 
@@ -781,7 +930,7 @@ Source::Source(GSource* cast_item, GSourceFunc callback_func)
 {
   g_source_set_callback(
       gobject_, callback_func,
-      new SourceCallbackData(this), // our persistant callback data object
+      new SourceCallbackData(this), // our persistent callback data object
       &SourceCallbackData::destroy_notify_callback);
 }
 
@@ -800,6 +949,8 @@ Source::~Source()
     gobject_ = 0;
 
     g_source_unref(tmp_gobject);
+
+    // The constructor does not add this to extra_source_data. No need to erase.
   }
 }
 
@@ -941,7 +1092,11 @@ TimeoutSource::~TimeoutSource()
 bool TimeoutSource::prepare(int& timeout)
 {
   Glib::TimeVal current_time;
+#ifndef GLIBMM_DISABLE_DEPRECATED
   get_current_time(current_time);
+#else
+  time64_to_time_val(get_time(), current_time);
+#endif //GLIBMM_DISABLE_DEPRECATED
 
   Glib::TimeVal remaining = expiration_;
   remaining.subtract(current_time);
@@ -978,7 +1133,11 @@ bool TimeoutSource::prepare(int& timeout)
 bool TimeoutSource::check()
 {
   Glib::TimeVal current_time;
+#ifndef GLIBMM_DISABLE_DEPRECATED
   get_current_time(current_time);
+#else
+  time64_to_time_val(get_time(), current_time);
+#endif //GLIBMM_DISABLE_DEPRECATED
 
   return (expiration_ <= current_time);
 }
@@ -989,7 +1148,11 @@ bool TimeoutSource::dispatch(sigc::slot_base* slot)
 
   if(again)
   {
+#ifndef GLIBMM_DISABLE_DEPRECATED
     get_current_time(expiration_);
+#else
+    time64_to_time_val(get_time(), expiration_);
+#endif //GLIBMM_DISABLE_DEPRECATED
     expiration_.add_milliseconds(std::min<unsigned long>(G_MAXLONG, interval_));
   }
 
